@@ -1,3 +1,11 @@
+from pkg_resources import parse_version
+
+try:
+    from functools import lru_cache
+except ImportError:  # pragma: no cover
+    from backports.functools_lru_cache import lru_cache
+
+
 APP_VERSION_2_API_VERSION_MAP = {
     "v4.1.0": "2.0",
     "v4.1.1": "2.0.1",
@@ -34,10 +42,18 @@ APP_VERSION_2_API_VERSION_MAP = {
     "v4.4.3.1": "2.8.5",
     "v4.4.4": "2.8.5",
     "v4.4.5": "2.8.5",
+    "v4.5.0beta1": "2.8.14",
+    "v4.5.0": "2.8.18",
 }
 
-MOST_RECENT_SUPPORTED_APP_VERSION = "v4.4.5"
-MOST_RECENT_SUPPORTED_API_VERSION = "2.8.5"
+MOST_RECENT_SUPPORTED_APP_VERSION = "v4.5.0"
+MOST_RECENT_SUPPORTED_API_VERSION = "2.8.18"
+
+
+@lru_cache(maxsize=None)
+def v(version):
+    """Caching version parser."""
+    return parse_version(version)
 
 
 class Version(object):
